/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill;

import anthill.util.Clearable;

/**
 *  Factory interface for creating ants. This interface is used by
 *  nest to create ants which will be used to satisfy requests
 *  in the Anthill network.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.8 $
 */
public interface AntFactory
extends Clearable
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Identifier used in the configuration files */
  public static final String NAME = "AntFactory";

 
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Factory method for creating a set of ants aimed at satisfying 
   *  a particular request. Multiple ants created for the same 
   *  request are completely independent. The interpretation of the 
   *  request is left to ant implementations.
   *
   *  @param request the object containing the request to be satisfied
   *  @throws IllegalArgumentException if the ants generated by this
   *    factory are not capable to deal with the specified request.
   */
  public Ant[] getAnts(RequestId rid, Object request, NestId requestor)
    throws IllegalArgumentException;

  
} // END AntFactory


