/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;

/**
 *  Interface <CODE>Closeness</CODE> is implemented by objects whose
 *  content can be compared against two other objects for closeness.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.1 $
 */
public interface Closeness
{

  /**
   *  Compares <code>this</code> object against the provided objects for
   *  closeness.  An implementation of this method must return an
   *  <code>int</code> according to the following rules:
   *
   *  <pre>
   *    -1 is returned if the value of x is considered closer to this object than the value of y.
   *    +1 is returned if the value of y is considered closer to this object than the value of x.
   *     0 is returned if the values of x and y are equally close to this object.
   *  </pre>
   */
  public int isCloser(Object x, Object y);

}
