/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;



/**
 *  This interface represents an hashed keyword, used in ant algorithms based
 *  on hash routing. An hashed keyword contains two values, the keyword 
 *  maintained as a plain-text string, and the hash value maintained as
 *  an array of bytes.
 *
 *  @author Alberto Montresor (montresor@cs.unibo.it)
 *  @version $Revision: 1.2 $
 */
public interface HashedKeyword 
extends Comparable, Closeness
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  public static final String NAME = "HashedKeyword";
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Initializes the hashed keyword by setting the keyword contained in it.
   */
  public void init(String keyword);
  
  /**
   *  Returns the keyword associated with this hashed keyword.
   */
  public String getKeyword();

  /**
   *  Returns the bytes representing the hashed value of the keyword.
   */
  public byte[] getBytes();

} // END HashedKeyword
