/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;

import java.util.Random;

/**
 *  
 *
 *  @author Alberto Montresor (montresor@cs.unibo.it)
 *  @version $Revision: 1.1 $
 */
public class SimpleDocumentId 
implements DocumentId
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Random generator */
  private static Random random = new Random();

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Document length */
  private int length; 
  
  /** Document digest */
  private int digest;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  public SimpleDocumentId()
  {
    length = Math.abs(random.nextInt());
    digest = random.nextInt();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////


  /**
   * Returns the length of the document
   */
  public int getLength()
  {
    return length;
  }

  /**
   * Returns the byte representation of the digest associated to this document
   * identifier.
   */
  public byte[] getDigest()
  {
    byte[] ret = new byte[4];
    // FIXME Complete this implementation
    return ret;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Returns a string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("[SimpleDocId: ");
    buf.append(digest);
    buf.append(", ");
    buf.append(length);
    buf.append("]");
    return buf.toString();
  }
  
  /**
   *  Check whether this hashed keyword is equal to the specified object.
   */
  public boolean equals(Object obj)
  {
    try {
      SimpleDocumentId id = (SimpleDocumentId) obj;
      if (id.digest == this.digest && id.length == this.length) 
        return true;
    } catch (Exception e) { }
    return false;
  }
  
  /**
   *  Returns a hash code value for the object.
   */
  public int hashCode()
  {
    return digest ^ length;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Comparable
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Compares this object with the specified object for order. Returns a 
   *  negative integer, zero, or a positive integer as this object is less 
   *  than, equal to, or greater than the specified object. 
   *  
   *  @param obj the object to be compared
   */
  public int compareTo(Object obj)
    throws ClassCastException
  {
    SimpleDocumentId docId = (SimpleDocumentId) obj;
    if (docId.digest < this.digest) {
      return -1;
    } else if (docId.digest == this.digest) {
      if (docId.length < this.length)
        return -1;
      else if (docId.length == this.length) 
        return 0;
      else 
        return 1;
    } else {
      return 1;
    }
  }
  
} // END SimpleDocumentId
