/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.util;

import java.util.Random;

/**
 *  This class extends <CODE>Random</CODE> by adding some utility methods.
 *  These additions include methods for tossing coins with a specified
 *  probability, and to obtain random value with normal distribution
 *  with specified mean and deviation.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class ExtendedRandom
extends Random
{

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  // Javadoc comment inherited from superclass
  public ExtendedRandom()
  {
    super();
  }
  
  // Javadoc comment inherited from superclass
  public ExtendedRandom(long seed)
  {
    super(seed);
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods 
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  
   */
  public int nextNormal(int mean, double deviation)
  {
    return Math.round( (float) nextNormal( (double) mean, deviation) );
  }

  /**
   *  
   */
  public int nextAbsoluteNormal(int mean, double deviation)
  {
    return Math.round( (float) nextAbsoluteNormal( (double) mean, deviation) );
  }

  /**
   * 
   */
  public float nextNormal(float mean, double deviation)
  {
    return (float) nextNormal((double) mean, deviation);
  }
  
  /**
   * 
   */
  public float nextAbsoluteNormal(float mean, double deviation)
  {
    return (float) nextAbsoluteNormal((double) mean, deviation);
  }

  /**
   * 
   */
  public double nextNormal(double mean, double deviation)
  {
    return mean + (nextGaussian() * deviation);
  }

  /**
   * 
   */
  public double nextAbsoluteNormal(double mean, double deviation)
  {
    return mean + Math.abs((nextGaussian() * deviation));
  }

  /**
   *  Toss a coin returning true or false, with the specified
   *  probability of returning true.
   * 
   *  @param prob The probability of returning true
   */
  public boolean tossCoin(float prob)
  {
    float coin = nextFloat();
    return (coin < prob);
  }
  
} // END ExtendedRandom


