/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.jxtagnutant;

import java.io.Serializable;

import anthill.*;

/**
 *
 *  @author Hein Meling
 *  @author Francesco Russo (frusso@cs.unibo.it)
 *  @version $Revision: 1.4 $
 */
public class ReplyGnutant
implements Ant, Serializable
{

  private static java.util.Random rand;
  static 
  {
  	rand = new java.util.Random();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The identifier of the request */
  private RequestId rid;

  /** Resource to be returned to the requestor nest */
  private Resource[] resources;

  /** Identifier of the requestor nest */
  private NestId requestor;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Default constructor for externalization
   */
  public ReplyGnutant()
  {
	System.out.println("--- ReplayGnutant: created!");
  }

  /**
   *  Initializes a new ReplyGnutant with the set of resources to be returned,
   *  the request identifier and the identifier of the requestor nest.
   */
  public ReplyGnutant(RequestId rid, Resource[] resources, NestId requestor)
  {
  	this.rid = rid;
  	this.resources = resources;
  	this.requestor = requestor;
  	System.out.println("--- ReplayGnutant: created!");
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  // Documentation inherited by interface
  public String getSpecies()
  {
  	return "jxtagnutant";
  }

  public void init(AntView view)
  {
  }

  /**
   *  This method encapsulate the computation performed by the ant.
   *
   *  @param view reference to the AntView object which
   *    constitutes the controlled environment of the ant
   */
  public void run(AntView view)
  {
	System.out.println("--- ReplayGnutant: run()");
  	//if (!requestor.equals(view.getNestId()))
  	if ( ! requestor.toString().trim().toLowerCase().equals(view.getNestId().toString().trim().toLowerCase()) )
  	{
		System.out.println("--- ReplayGnutant: run() - NOT at home"); 
		try 
		{
	  		System.out.println("--- ReplayGnutant: run() - about to move");
			view.move(requestor);
			System.out.println("--- ReplayGnutant: run() - done!");
		} 
		catch (Exception e) 
		{
			// Do nothing; the requestor is not there
			//Log.print(200, "@"); // This should never happen in the simulation!
			move( view );
		}
  	} 
  	else 
  	{
		System.out.println("--- ReplayGnutant: run() - FINALLY back home :)!"); 
		/* We are in the requestor nest */
		view.returnResources(rid, resources);
		System.out.println("--- ReplayGnutant: run() - resources returned :)!");
  	}
  }


  /**
  *  This private method moves randomly the invoking ant to another nest taken from 
  *  the current nest's list of known neighbours. If this list was empty or no nests were
  *  reachable the ant would wait indefinitely until a new neighbour becomes available.
  */
  private void move(AntView view)
  {
  	Neighbour[] neighbours = view.getNeighbours();
	  	 
	while (neighbours.length < 1)
	{
		try
	  	{
			Thread.sleep(4000);
	  	}
	  	catch(InterruptedException ie)
	  	{
	        ie.printStackTrace();
		}
   		neighbours = view.getNeighbours();
	}
	int next = rand.nextInt(neighbours.length);
	try 
	{
		System.out.println("--- ReplyGnutant: move() - goForward - moving on, nothing has been found!");
		view.move(neighbours[next].getNestId());
	} 
	catch (Exception e) 
	{
		System.out.println("--- ReplyGnutant: move() - nest unreachable, looking for a new one!");
		move( view );
	}   
  }



  ////////////////////////////////////////////////////////////////////////////////////////////
  // Externalization methods
  ////////////////////////////////////////////////////////////////////////////////////////////

} // END ReplyGnutant
