/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import anthill.*;
import anthill.storage.Storage;
import anthill.util.Clearable;

/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.7 $
 */
public interface MessorPheromone
extends Clearable, Storage
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Identifier used in the configuration files */
  public static final String NAME = "MessorPheromone";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods 
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Add the specified node and its associated load to the pheromone table.
   */
  public void addNest(NestId id, int load);
    
  /**
   *  Returns the next nest to be followed, depending on
   *  some parameters and on the content of the storage.
   */
  public NestId getNextMaxNest(double deviation);
  
  /**
   * Returns the next nest to be followed, depending on
   * some parameters and on the content of the storage.
   */
  public NestId getNextMinNest(double deviation);

  /**
   * 
   */
  public NestId getRandomNest();
  
  
  /**
   * 
   */
  public void update(NestId local, NestId[] nests, int[] loads);
    
  /**
   * 
   */
  public void update(NestId local, Neighbour[] nests);

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Visit management methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  If the number of visits is smaller than the minimum amount of visits to be
   *  received, returns the number of missing visits. Return 0 otherwise. 
   */
  public boolean isUnderloaded();
  
  /**
   *  Returns true if the node is overloaded by ants, that have to be removed
   *  from the nest.
   */
  public boolean isOverladed();
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Statistics methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Add the nest identifiers contained in the nest to
   *  the set identified
   */
  public PheromoneEntry[] getEntryTable();
  
} // END MessorPheromone

