/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import anthill.NestId;

/**
 *  Data structure containing the identifier of a nest and the load 
 *  associated to it.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class PheromoneEntry
implements Comparable
{
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object 
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Nest identifier */
  NestId id;

  /** Load associated to the nest */
  int load;

  /** Time of insertion */
  int time;

  /** Index in the table */
  int index;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object 
  ////////////////////////////////////////////////////////////////////////////////////////////

  public PheromoneEntry(NestId id, int load, int time)
  {
    this.id = id;
    this.load = load;
    this.time = time;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object 
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return the identifier of the nest contained in this entry. 
   */
  public NestId getNestId()
  {
    return id;
  }
  
  /**
   *  Return the load of the nest contained in this entry. 
   */
  public int getLoad()
  {
    return load;
  }

  /**
   *  Return the time in which this entry has been inserted in the pheromone table. 
   */
  public int getTime()
  {
    return time;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object 
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * 
   */
  public boolean equals(Object obj) 
  {
    //Log.println("equals: this=" + this + ", obj=" + obj);
    if (obj != null && obj instanceof PheromoneEntry) {
      PheromoneEntry entry = (PheromoneEntry) obj;
      return (this.id == entry.id);
    } else {
      return false;
    }
  }
  
  public int hashCode()
  {
    return id.hashCode();
  }
  
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    buffer.append("[id=");
    buffer.append(id);
    buffer.append(", load=");
    buffer.append(load);
    buffer.append(", time=");
    buffer.append(time);
    buffer.append("]");
    return buffer.toString();
  }
    
  public int compareTo(Object o2)
  {
    //Log.println("loadComparator: o1=" + o1 + ", o2=" + o2);
    PheromoneEntry e1 = (PheromoneEntry) this;
    PheromoneEntry e2 = (PheromoneEntry) o2;
    int res;
    if (e1.load < e2.load) {
      res = -1;
    } else if (e1.load == e2.load) {
      // FIXME Currently, only LocalNid are comparable;
      Comparable obj = (Comparable) e1.id;
      return obj.compareTo(e2.id); 
    } 
    return 1;
  }
  
  
} // END PheromoneEntry

