/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim;

import anthill.util.Clearable;

/**
 *  Classes implementing this interface are used to generate requests
 *  during a simulation.
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public interface RequestGenerator
extends Clearable
{
 
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Identifier used in the configuration files */
  public static final String NAME = "RequestGenerator";
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Generates a set of requests in the specified P2P network.
   *  Different implementations may adopt different generation and
   *  distribution patterns.
   */
  public void generateRequests();

  public void init(ResourceSet resourceSet, Peernet peernet, ResponseMonitor monitor);

  
} // END RequestGenerator

