/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import anthill.factory.Factory;
import anthill.util.Log;
import antsim.*;

/**
 *  This class implements the <CODE>ScenarioAnalyzer</CODE> interface; its
 *  task is to print the content of the statistics object whenever the
 *  <CODE>analyze</CODE> method is invoked.
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class GenericScenarioAnalyzer
implements ScenarioAnalyzer
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  True if the heading of the statistics have already been printed;
   *  false otherwise. Initially, the value is set to false.
   */
  private boolean heading;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Instantiates a new GenericScenarioAnalyzer. This implementation
   *  simply prints out the 
   *
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GenericScenarioAnalyzer(Factory factory, String name)
  {
    clear();
  }
  
  
  /**
   * Restores the object to its initial state.
   */
  public void clear()
  {
    heading = false;
  }

    
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Performs an analysis on the state of the simulated P2P network.
   *  This implementation simply prints out the statistics collected 
   *  at runtime in the statistics object
   *  
   *  @param stats the object used to collect statistics
   *  @param peernet the simulated P2P network, whose content can
   *    be inspected by accessing the storage objects included in it.
   */
  public void analyze(Stats stats, Peernet peernet)
  {
    if (!heading) {
      Log.print(200, stats.headingToString());
      heading = true;
    }
    Log.print(200, stats.toString());
    
  }
  
} // END GenericScenarioAnalyzer

