/* Generated by Together */

package jxtaimpl;

import java.io.IOException;

import anthill.*;
import anthill.factory.Factory;
import anthill.nest.AntDescriptor;
import anthill.storage.*;

/**
 * This class is the enviroment where an Ant can operate.
 * @author Francesco Russo (frusso@cs.unibo.it)
 */
public class AntView implements anthill.AntView
{   
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for requesting general information
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
	 *  Default constructor. The nest object is the <CODE>Nest</CODE> over which the 
	 *  <CODE>Ant</CODE> wrapped in the <CODE>AntDescriptor</CODE> will operate
	 *
	 * @param nest The <code>Nest</code> this <code>AntView</code> belongs to
	 * @param aDesc The <code>AntDescriptor</code> wrapping the <code>Ant</code>
	 * @param factory The <code>Factory</code> instance used for reading in configuration
	 *         parameters
	 */
  public AntView(Nest nest, AntDescriptor aDesc, Factory factory) 
  {
	this.nest = nest;
	this.aDesc = aDesc;
	this.factory = factory;
  }

  /**
	 *  Returns the identifier of the nest hosting the
	 *  ant using this <CODE>AntView</CODE>.
	 */
  public NestId getNestId()
  {
	return nest.getNestId();
  }
  
  /**
	 *  Returns the identifier of the last nest visisted by this
	 *  ant.
	 */
  public NestId getLastVisited()
  {
	if ( aDesc.getLastVisited() == null )
	{
	  System.out.println("--- AntView: getLastVisited() NESTID is NULL!");
	  System.exit(1);
	}
	return aDesc.getLastVisited();
  }
  
  /**
   * Returns this ant's TTL value
   */
  public int getTTL()
  {
	return aDesc.getTTL();
  }

  /**
   *  Return the ant which is associated to this <CODE>AntView</CODE>. Normally not
   *  used by ants, as they may access the same information using
   *  <CODE>this</CODE>.
   */
  public Ant getAnt()
  {
	return aDesc.getAnt();
  }
  
  /**
  *  Creates a new ant identifier.
  */
  public AntId createAntId()
  {
	RequestId rId = new jxtaimpl.ids.RequestId( nest.getNestId() );
	AntId aId = new jxtaimpl.ids.AntId( rId );
  
	return aId;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for ant movement and creation
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Add a new ant to this nest. For example, this method
   *  could be used to create a backward ant for returning
   *  information to the requestor nest.
   *
   *  @param ant the ant to be added to the current nest
   */
  public void addAnt(Ant ant)
  {
	AntDescriptor newdesc = new AntDescriptor(nest.getNestId(), ant, aDesc.getTTL());
	AntView view = new AntView(nest, newdesc, factory);
	nest.getManager().add(view);
  }

  /** 
  *  Method invoked by an ant in order to be sent to another 
  *  <CODE>Nest</CODE>. It is possible that particular 
  *  <CODE>AntView</CODE> implementation limit the operations 
  *  which can be performed after a move operation. For example, 
  *  movement to more than one nest (i.e., duplication) may not
  *  be enabled.
  *
  *  @param id the identifier of the destination nest
  *  @throws IOException raised if the move operation fails,
  *    for example due to the unreachability of the nest
  */
  public void move(NestId id) throws IOException
  {
   if ( id == null )
	{
	  System.out.println("--- AntView: move() NESTID is null!");
	  System.exit(1);
	} 
   nest.move( id, aDesc.getAnt(), aDesc.getTTL() );
  }
  
	
  // FIXME (X FRANCESCO): TO BE REMOVED
  /**
	 *  Store information about the last nest visited by the ant
	 *  associated with this <CODE>AntView</CODE> on the current 
	 *  nest.  This information is constituted of a pair
	 *  (<CODE>antId</CODE>, <CODE>nestId</CODE>) where 
	 *  <CODE>antId</CODE> is the identifier of the ant and
	 *  <CODE>nestId</CODE> is the identifier of the nest from
	 *  which the ant has arrived. During the return trip, this
	 *  information will be used to obtain the identifier of the 
	 *  previous nest in the incoming path.
	 *  
	 *  @return <CODE>true</CODE> if this is the first visit of this ant
	 *    to this nest (and thus, the information about the previous nest
	 *    has been stored successfully), or <CODE>false</CODE> if this
	 *    ant has already visited the nest (and thus, the inforamtion about
	 *    the previous nest could not be stored).
	 */
/*
  public boolean storeReturnNest(AntId aid, NestId nid)
  {
	Trail trail = nest.getTrail(true);
	return trail.add(aid, aDesc.getLastVisited());
  }
*/
  // FIXME (X FRANCESCO): TO BE REMOVED
  /**
	 *  Returns the identifier of the previous nest in the incoming path,
	 *  as stored through during the forward trip through method  
	 *  <CODE>storePreviousNest()</CODE>. If the current nest is the requestor
	 *  nest (i.e., the nest which has created this ant), the method returns
	 *  <CODE>null</CODE>.
	 *  
	 *  @param remove true if the association (ant id, nest id)
	 *    should be removed after retrieval
	 *  @throws IllegalStateException raised if no information about the previous
	 *    nest visited by this ant is present (for example, due to expiration).
	 */
/*
  public NestId getReturnNest(AntId aid, boolean remove)
  {
	Trail trail = nest.getTrail(false);
	if (trail != null) 
	{
	 return trail.lookup(aid, remove);
	} 
	else 
	{
	 throw new IllegalStateException("No trail information in this nest");
	}
  }
*/
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for neighbour management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
	 *  Ants invoke this method to add the identifier of a new neighbour
	 *  nest to the set of neighbours known to the current nest. Depending
	 *  on the implementation, this operation may require the establishment
	 *  of a connection to the new nest. In this case, an <CODE>IOException</CODE> 
	 *  may be raised in the case a connection cannot be established.
	 *
	 *  @throws IOException if a connection with the nest cannot be established.
	 */
  public void addNeighbour(NestId id)
  throws IOException
  {
	nest.addAntNeighbour(id);
  }
  
  /**
  *  Ants invoke this method to remove a neighbor nest from the list of
  *  neighbors known to this nest. 
  */
  public void removeNeighbour(NestId id)
  {
	nest.removeNeighbour( id );
  }
  /**
   *  Ants invoke this method to obtain the list of nest neighbors
   *  known to the current nest.
   */
  public Neighbour[] getNeighbours()
  {
	return nest.getNeighbours();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for storage management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return a storage object identified by the given name.
   *  The storage we return with this method, has to be the storage associated
   *  with the invoking ant's species.
   *  Name is used for specifieing the storage type (i.e. Storage.DOC_STORAGE,
   *   Storage.PHR_STORAGE, Storage.URL_STORAGE)
   */
  public ResourceManager getResourceManager(String name)
  {
	return nest.getResourceManager( name, aDesc.getAnt().getSpecies() );
  }
  
  /**
   *  Return a storage object identified by the given name.
   *  The storage we return with this method, has to be the storage associated
   *  with the invoking ant's species.
   *  Name is used for specifieing the storage type (i.e. Storage.DOC_STORAGE,
   *   Storage.PHR_STORAGE, Storage.URL_STORAGE). 
   */
  public ResourceManager createResourceManager(String name) 
	throws ResourceManagerException
  {
	return nest.createResourceManager( name, aDesc.getAnt().getSpecies() );
  }
	
  /**
   *  Get the pheromone information object related to this ant.
   *  The pheromone object is the one associated with the invoking ant's species.
   *  Remember to invoke the <code>updatePheromone(Object pheromone)</code>
   *  method in order to commit changes performed by ants to the pheromone 
   *  instace.
   */
/* 
  public Object getPheromone()
  {
	return nest.getPheromone( aDesc.getAnt().getSpecies() );
  }
*/
  /**
   *  Set the pheromone information object related to this ant.
   *  Remember to invoke the <code>updatePheromone(Object pheromone)</code>
   *  method in order to commit changes performed by ants to the pheromone 
   *  instace.
   *
   *  @throws CapacityException if the nest cannot store the
   *    the pheromone object.
   */
/*
  public void setPheromone(Object pheromone)
  {
	nest.setPheromone( aDesc.getAnt().getSpecies(), pheromone );
  }
*/  
  /**
   *  This method should be invoked by ants in order to update the current
   *  nest's pheromone instance every time they modify it. Not invokig this
   *  method implies having inconsisten information stored in a nest's pheromone
   *  instance and so a wrong system behaviour.
   */
  public void updatePheromone(Object pheromone)
  {
	nest.updatePheromone( aDesc.getAnt().getSpecies(), pheromone );
  }
  
  /**
   *  Performs the specified request on the local resource
   *  storage and returns the resources satisfying the
   *  request. 
   *  @param request The request to be performed
   */
  public Resource[] getResources(Object query) throws StorageNotFoundException
  {
	return nest.getResources(query, aDesc.getAnt().getSpecies());
  }

  /**
   *  Performs the specified request on the local resource
   *  storage and returns the resources satisfying the
   *  request. 
   *
   *  @param request The request to be performed
   */
  public Resource getResource(Object query) throws StorageNotFoundException
  {
	System.out.println("--- AntView: getResource()");
	return nest.getResource(query, aDesc.getAnt().getSpecies());
  }
  
  /**
   *  Add a resource to the nest. This method enables
   *  ants to move resources from one nest to another. 
   *  This method will copy the resource into the local <CODE>Storage</CODE>
   *  associated with the invoking ant's species.
   *
   *  @param resource resource to be added
   */
  public void addResource(Resource resource) throws CapacityException, StorageNotFoundException
  {
	nest.insert(resource, aDesc.getAnt().getSpecies());
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for resource management
  ////////////////////////////////////////////////////////////////////////////////////////////


  /**
   *  This method is invoked by ants to notify the nest that
   *  the ant associated to this <CODE>AntView</CODE>, created to satisfy
   *  a request, has found the specified set of resources.
   *
   *  @param resources  the resources satisfying the request
   */
  public boolean returnResources(RequestId rid, Resource[] resources)
  {
	return nest.getRequestSet().returnResources(rid, resources);
  }
  
  
  /**
   * Read the configuration parameter from its own <code>Factory</code> instance.
   */
  public int intValue(String name)
  {
	return factory.intValue( name );
  }

  private Nest nest;
  private AntDescriptor aDesc;
  private Factory factory;
}





