/* Generated by Together */

package jxtaimpl;

import java.io.*;
import java.net.*;
import java.util.*;

import net.jxta.discovery.DiscoveryService;
import net.jxta.document.*;
import net.jxta.exception.PeerGroupException;
import net.jxta.id.IDFactory;
import net.jxta.impl.peergroup.StdPeerGroupParamAdv;
import net.jxta.peergroup.*;
import net.jxta.pipe.*;
import net.jxta.platform.*;
import net.jxta.protocol.*;

import anthill.*;
import anthill.factory.Factory;


/**
 * This class implements the PeerFinder interface. So, after having initialized and started the Nest, its instance will periodically search for new advertisements.
 * @author Francesco Russo (frusso@cs.unibo.it) 
 */
public class AnthillPeerWS extends Thread implements PeerFinder 
{
  /**
   * The constructor needs a Factory in order to instantiate all the other 
   * components of the Nest. 
   *
   * @param factory This is the factory instance used for reading in 
   *				configuration parameters
   */
  public AnthillPeerWS( Factory factory ) 
  {
	//* settings for the PeerFinder and the WS
	super("AthillGroup-WS-PeerFinder");
	
	System.setSecurityManager(new SecurityManager() );
	
	this.factory = factory;
	sleepTime = factory.intValue( SLEEP_TIME );
	this.setPriority( Thread.MIN_PRIORITY );
	//confDir = factory.stringValue( CONF_DIR );
  
	try
	{
	  //* JXTA settings 
	  ANTHILL_GROUP_ID = (net.jxta.peergroup.PeerGroupID) IDFactory.fromURL( new URL("urn","","jxta:uuid-1E7C50EAFF04430CBAF780BEC7B9E98902") );
	  ANTHILL_GROUP_MSPEC_ID = (net.jxta.platform.ModuleSpecID) IDFactory.fromURL( new URL("urn","","jxta:uuid-DEADBEEFDEAFBABAFEEDBABE00000001856BDDB638074FCF8DD9B1DEEB387EDC06") ); 
	  ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID = (net.jxta.platform.ModuleClassID) IDFactory.fromURL( new URL("urn","","jxta:uuid-DC4D650F4167455B900196015072C86805") );
	  ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID = (net.jxta.platform.ModuleSpecID) IDFactory.fromURL( new URL("urn","","jxta:uuid-DC4D650F4167455B900196015072C8683E708249C9044061B827929D2333FBC506") );
	  ANTHILL_GROUP_DOWNLOAD_SERVICE_MCLASS_ID = (net.jxta.platform.ModuleClassID) IDFactory.fromURL( new URL("urn","","jxta:uuid-2B02EA08671D4989A0B89DF413E5610C05") );
	  ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID = (net.jxta.platform.ModuleSpecID) IDFactory.fromURL( new URL("urn","","jxta:uuid-2B02EA08671D4989A0B89DF413E5610C81AD7ECD6CFB4C48B73DB7A6B12DB01806") );
	}
	catch(MalformedURLException mue)
	{
	  mue.printStackTrace();
	  System.exit(1);
	}
	catch(java.net.UnknownServiceException use)
	{
	  use.printStackTrace();
	  System.exit(1);
	}
  
	//* now we set the whole local nest up
	initialize();
	
	//* now we have to start the PeerFinder thread
	this.start();
  }
  
  /**
  * This method returns the <CODE>Nest</CODE> instance created by the <CODE>
  * AnthillPeerWorkSpace</CODE>.
  * This method should be used by an application developed on the top of this
  * Anthill implementation in order to obtain a reference to the local Nest.
  */
  public Nest getNest()
  {
	return nest;
  }

  /**
   * Needed to set up the platform. 
   */
  private PeerGroup startJxta() 
  {
	PeerGroup netGroup = null;
	
	try
	 {
	  //create, and Start the default jxta NetPeerGroup
	  netGroup = PeerGroupFactory.newNetPeerGroup();
	 }
	catch (PeerGroupException pge)
	 {
	  pge.printStackTrace();
	  System.exit(1);
	 }
	catch (Exception e)
	 {
	  e.printStackTrace();
	  System.exit(1);
	 } 
	return netGroup;
  }
  
  /**
  * With this method we obtain the references to the Discovery and the Pipe services of the NetPeerGroup
  * useful for publishing the GateServicePipeAdv
  */
  private void setServices(PeerGroup grp)
  {
	netDiscoService = grp.getDiscoveryService();
	netPipeService = grp.getPipeService();
	if ((netDiscoService!=null)&(netPipeService!=null)) System.out.println("--- AnthillPeerWS - Got services!");
  }

  private void joinGroup(AnthillGroup grp) 
  {
  }

  /**
  * This method returns the local nest ID
  */
  public NestId getNestId()
  { 
	return localNestId; 
  }

  /**
  * This method generates the GateService pipe adv which has to be published in the NetPeerGroup
  * in order to be discovered by other nests. The ANTHILL_GROUP_ID has to be supplied for a valid
  * PipeID
  */
  private PipeAdvertisement createGateServicePipeAdvertisement(PeerGroupID id) 
  {
	PipeAdvertisement pipeAdv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());
	  
	pipeAdv.setName( ANTHILL_GROUP_GATE_SERVICE_PIPE_ADV_NAME );
	PipeID pipeId = createPipeID( id );
	if ( pipeId != null )
	{
	  pipeAdv.setPipeID( pipeId );
	  LOCAL_GATE_SERVICE_PIPE_ID = pipeId;
	}
	else
	{
	  System.out.println("--- AnthillPeerWorkSpace: pipeId is null, giving up!");
	  System.exit(1);
	}
	pipeAdv.setType( PipeService.UnicastType );
	return pipeAdv;
  }

  /**
  * This method generates the DownloadService pipe adv. The ANTHILL_GROUP_ID has to be supplied 
  * in order to create a valid PipeID
  */
  private PipeAdvertisement createDownloadServicePipeAdvertisement(PeerGroupID id) 
  {
	PipeAdvertisement pipeAdv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());
	  
	pipeAdv.setName( ANTHILL_GROUP_DOWNLOAD_SERVICE_PIPE_ADV_NAME );
	PipeID pipeId = IDFactory.newPipeID( id );
	if ( pipeId != null )
	{
	  pipeAdv.setPipeID( pipeId );
	}
	else
	{
	  System.out.println("--- AnthillPeerWorkSpace: download pipeId is null, giving up!");
	  System.exit(1);
	}
	pipeAdv.setType( PipeService.UnicastType );
	return pipeAdv;
  }

  /**
  * This private method read the PipeId from file if it does exists, otherwise
  * a new PipeId will be instantiated and saved to disk.
  */
  private net.jxta.pipe.PipeID createPipeID( PeerGroupID grpId )
  {
	pipeIdFile = new File( anthillDir, PIPE_ID_FILE_NAME );
	net.jxta.pipe.PipeID id;
	
	if ( pipeIdFile.exists() )
	{
	  try
	  {
		FileInputStream fis = new FileInputStream( pipeIdFile );
		ObjectInputStream ois = new ObjectInputStream( fis );
		URL idAsURL = (URL) ois.readObject();
		fis.close();
		id = (net.jxta.pipe.PipeID) IDFactory.fromURL( idAsURL );
		return id;
	  }
	  catch(IOException ioe)
	  {
		ioe.printStackTrace();
		System.out.println("--- AnthillPeerWorkSpace: could not read file "+PIPE_ID_FILE_NAME+", giving up!");
		System.exit(1);
	  }
	  catch(ClassNotFoundException cnfe)
	  {
		cnfe.printStackTrace();
		System.out.println("--- AnthillPeerWorkSpace: could not read file "+PIPE_ID_FILE_NAME+", giving up!");
		System.exit(1);
	  }
	}
	//* the PipeId will be written as URL, since the class PipeID does not
	//* implement Serializable
	id = IDFactory.newPipeID( grpId );
	try
	{
	  URL idAsURL = id.getURL();  
	  FileOutputStream fos = new FileOutputStream( pipeIdFile );
	  ObjectOutputStream oos = new ObjectOutputStream( fos );
	  oos.writeObject( idAsURL );
	  fos.close();
	}
	catch(IOException ioe)
	{
	  ioe.printStackTrace();
	  System.out.println("--- AnthillPeerWorkSpace: could not write file "+PIPE_ID_FILE_NAME);
	  System.out.println("                          there might be problems at the next start up!");
	}
	return id;
  }

  /**
   * This method generates a ModuleSpecAdvertisement for the GateService implemented by the AnthillGroup.
   * The ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID is needed
   */
  private ModuleSpecAdvertisement createGateServiceModSpecAdv(ModuleSpecID moduleSpecID, PipeAdvertisement pipeAdv) 
  {
	ModuleSpecAdvertisement modSpecAdv = (ModuleSpecAdvertisement) AdvertisementFactory.newAdvertisement( ModuleSpecAdvertisement.getAdvertisementType() );
	  
	modSpecAdv.setCreator( ANTHILL_GROUP_GATE_SERVICE_MSPEC_CREATOR );
	modSpecAdv.setDescription( ANTHILL_GROUP_GATE_SERVICE_MSPEC_DESCRIPTION );
	modSpecAdv.setModuleSpecID( ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID );
	modSpecAdv.setName( ANTHILL_GROUP_GATE_SERVICE_MSPEC_NAME );
	modSpecAdv.setSpecURI( ANTHILL_GROUP_GATE_SERVICE_MSPEC_URI );
	modSpecAdv.setVersion( ANTHILL_GROUP_GATE_SERVICE_MSPEC_VERSION );
	modSpecAdv.setPipeAdvertisement( pipeAdv );
	  
	return modSpecAdv;
  }

  /**
   * This method generates a ModuleImplAdvertisement for the GateService implemented by the AnthillGroup.
   * The ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID is needed
   */
  private ModuleImplAdvertisement createGateServiceModImplAdv(ModuleSpecID moduleSpecID) 
  {
	ModuleImplAdvertisement modImplAdv = (ModuleImplAdvertisement) AdvertisementFactory.newAdvertisement( ModuleImplAdvertisement.getAdvertisementType() );
	  
	modImplAdv.setCode( ANTHILL_GROUP_GATE_SERVICE_MIMPL_CODE );
	modImplAdv.setDescription( ANTHILL_GROUP_GATE_SERVICE_MSPEC_DESCRIPTION );
	modImplAdv.setModuleSpecID( moduleSpecID );
	modImplAdv.setProvider( ANTHILL_GROUP_GATE_SERVICE_MSPEC_URI );
	modImplAdv.setUri( ANTHILL_GROUP_GATE_SERVICE_MSPEC_URI );
	  
	return modImplAdv;
  }

  /**
   * This method generates a ModuleSpecAdvertisement for the DownloadService implemented by the AnthillGroup.
   * The ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID is needed
   */
  private ModuleSpecAdvertisement createDownloadServiceModSpecAdv(ModuleSpecID moduleSpecID, PipeAdvertisement pipeAdv) 
  {
	ModuleSpecAdvertisement modSpecAdv = (ModuleSpecAdvertisement) AdvertisementFactory.newAdvertisement( ModuleSpecAdvertisement.getAdvertisementType() );
	  
	modSpecAdv.setCreator( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_CREATOR );
	modSpecAdv.setDescription( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_DESCRIPTION );
	modSpecAdv.setModuleSpecID( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID );
	modSpecAdv.setName( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_NAME );
	modSpecAdv.setSpecURI( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_URI );
	modSpecAdv.setVersion( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_VERSION );
	modSpecAdv.setPipeAdvertisement( pipeAdv );
	  
	return modSpecAdv;
  }

  /**
   * This method generates a ModuleImplAdvertisement for the DownloadService implemented by the AnthillGroup.
   * The ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID is needed
   */
  private ModuleImplAdvertisement createDownloadServiceModImplAdv(ModuleSpecID moduleSpecID) 
  {
	ModuleImplAdvertisement modImplAdv = (ModuleImplAdvertisement) AdvertisementFactory.newAdvertisement( ModuleImplAdvertisement.getAdvertisementType() );
	  
	modImplAdv.setCode( ANTHILL_GROUP_DOWNLOAD_SERVICE_MIMPL_CODE );
	modImplAdv.setDescription( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_DESCRIPTION );
	modImplAdv.setModuleSpecID( moduleSpecID );
	modImplAdv.setProvider( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_URI );
	modImplAdv.setUri( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_URI );
	  
	return modImplAdv;
  }

  /**
  * With this method we generate a ModuleImplAdvertisement for initializing the AnthillGroup.
  * The argument is the module impl adv taken from the NetPeerGroup
  */
  private ModuleImplAdvertisement createAnthillGroupModuleImplAdvertisement(ModuleImplAdvertisement modImplAdv) 
  {
	StdPeerGroupParamAdv paramAdv = null;
	  
	try
	 {
	  paramAdv = new StdPeerGroupParamAdv(modImplAdv.getParam());
	 }
	catch(PeerGroupException pge)
	 {
	  pge.printStackTrace();
	  System.exit(1);
	 }
	   
	gateServiceModImplAdv.setCompat( modImplAdv.getCompat() );
	downloadServiceModImplAdv.setCompat( modImplAdv.getCompat() );
	
	Hashtable hTab = paramAdv.getServices();
	hTab.put( ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID, gateServiceModImplAdv );
	hTab.put( ANTHILL_GROUP_DOWNLOAD_SERVICE_MCLASS_ID, downloadServiceModImplAdv );
	
	modImplAdv.setModuleSpecID( ANTHILL_GROUP_MSPEC_ID );
	modImplAdv.setParam( (Element)((StructuredDocument)paramAdv.getDocument(new MimeMediaType("text/xml"))));
	  
	return modImplAdv;
  }

  /**
   * This method should be invoked by a Nest instance in order to know to which 
   * Nest an ant wil be sent. The destination Nest will be choose randomly with
   * a uniform distribution amongst the known neighbours.
   * If there are no neighbours or if something goes  wrong, a <CODE>null</CODE> 
   * value will be returned.
   */
  public NestId getDestNestId()
  { 
	if ( neighbourManager.size() > 0 )
	{
	  NestId[] ids = (NestId[]) neighbourManager.toArray();
	  int dim = ids.length;
	  double interval = 1/dim;
	  double x = StrictMath.random();
	  
	  for(int i=0; i<dim; i++)
	  {
		if ( x < (interval*(i+1)) )
		{
		  return ids[i];
		}
	  }
	}
	return null;
  }

  /**
   * The run method will just periodically call the discoverNest() method and then go to sleep for a specified amount of time.
   * The "sleeping time" is set via the constructor of the class implementing this interface. 
   */
  public void run()
  { 
	while(true)
	{
	  System.out.println("--- AnthillPeerWS: run() follows discoverNest()");
	  discoverNest();
	}         
  }

  /**
   * When a new nest has been found, a call to this method will just record the new 
   * NestId associated to the nest. 
   * 
   * @param id The id of the nest we want to add to the set of known neighbours
   */
  public void addKnownNest(NestId id)
  { 
	synchronized( neighbourManager )
	{
	  System.out.println("--- AnthillPeerWS: addKnownNest()");
	  neighbourManager.addNeighbour( id );
	  try
	  {
		FileOutputStream fos = new FileOutputStream( fileNeighbourManager );
		ObjectOutputStream oos = new ObjectOutputStream( fos );
		oos.writeObject( neighbourManager );
		fos.close();
	  }
	  catch(IOException ioe)
	  {
		ioe.printStackTrace();
		System.out.println("--- Nest: could not save the NeighbourManager instance!");
	  }
	}
  }

  /**
   * This method is used for discovering new Nest instances.
   * 
   */
  public void discoverNest()
  { 
	System.out.println("--- AnthillPeerWS: discoverNest()");
	try
	 {
	  Enumeration enum = null;
	  NestId id = null;
	
	  netDiscoService.getRemoteAdvertisements( null, DiscoveryService.ADV, "Name", ANTHILL_GROUP_GATE_SERVICE_PIPE_ADV_NAME, 20, null);
	  this.sleep( sleepTime );
	  enum = netDiscoService.getLocalAdvertisements( DiscoveryService.ADV, "Name", ANTHILL_GROUP_GATE_SERVICE_PIPE_ADV_NAME);
	  
	  while ( (enum!=null) && enum.hasMoreElements() )
	  {
		System.out.println("--- AnthillPeerWS: discoverNest() inside the WHILE");
		Advertisement adv = (Advertisement) enum.nextElement();
		if ( adv instanceof PipeAdvertisement )
		  {
			System.out.println("--- AnthillPeerWS: discoverNest() inside the WHILE, is a PIPE ADVERTISEMENT");
			if ( ! (adv.getID().toString().equals(gateServicePipeAdv.getID().toString())) )
			{
			  //* add this new nestId just if it isn't my nest's id
			  id = this.mkNestId( adv );
			  this.addKnownNest( id );
			  netDiscoService.flushAdvertisements( adv.getID().toString(), DiscoveryService.ADV );
			}
		  }
	  }
	 }
	catch (IOException ioe)
	 {
	  ioe.printStackTrace();
	  System.exit(1);
	 } 
	catch (InterruptedException ie)
	 {
	  ie.printStackTrace();
	  System.exit(1);
	 }  
  }

  /**
   * When a new PipeAdvertisement is found, it is handled to this method to extract a valid NestId from it, since that's the smallest information we need to get connected to other nests. 
   *
   * @param adv The pipe advertisement from which we derive a valid <code>jxtaimpl.ids.NestId</code>
   *			instance
   */
  public jxtaimpl.ids.NestId mkNestId(Advertisement adv)
  { 
	jxtaimpl.ids.NestId id = new jxtaimpl.ids.NestId( ((PipeAdvertisement)adv) );
	return id;
  }


  /**
  * This private method is used for reading from file a prevously serialized
  * NeighbourManager instance if the <CODE>serialization</CODE> element in the
  * configuration file has value <CODE>yes</CODE> and if there is one, 
  * otherwise a new instance will be returned. 
  */
  private NeighbourManager createNeighbourManager()
  {
	if ( saveInfo.trim().toLowerCase().equals("yes") )
	{
	  //anthillDir = new File(System.getProperty("user.home")+File.separator+CMS_DIR_NAME);
	  anthillDir = new File( System.getProperty("user.dir")+File.separator+CMS_DIR_NAME );
	  if ( !anthillDir.exists() )
	  {
		//* the AnthillResources folder doesn't exists, let's create it
		anthillDir.mkdir();
		fileNeighbourManager = new File( anthillDir, NEIGHBOUR_MNG_FILE_NAME );
		try
		{
		  fileNeighbourManager.createNewFile();
		  FileOutputStream fos = new FileOutputStream( fileNeighbourManager );
		  ObjectOutputStream oos = new ObjectOutputStream( fos );
		  NeighbourManager mng = new NeighbourManager();
		  oos.writeObject( mng );
		  fos.close();
		  return mng;
		}
		catch(IOException ioe)
		{
		  ioe.printStackTrace();
		  System.out.println("--- AnthillPeerWorkSpace: could not create configurationfiles, giving up!");
		  System.exit(1);
		}
	  }
	  else
	  {
		String[] files = anthillDir.list();
		  
		for (int i=0; i < files.length; i++)
		{
		  if ( files[i].trim().toLowerCase().equals(NEIGHBOUR_MNG_FILE_NAME.trim().toLowerCase()) )
		  {
			//* instantiate the pheromoneHashTable deserializing this file's content
			try
			{
			  fileNeighbourManager = new File( anthillDir, NEIGHBOUR_MNG_FILE_NAME );
			  FileInputStream fis = new FileInputStream( fileNeighbourManager );
			  ObjectInputStream ois = new ObjectInputStream( fis );
			  NeighbourManager mng = (NeighbourManager) ois.readObject();
			  ois.close();
			  fis.close();
			  return mng;
			}
			catch(IOException ioe)
			{
			  ioe.printStackTrace();
			  System.out.println("--- AnthillPeerWorkSpace: could not read configuration file "+NEIGHBOUR_MNG_FILE_NAME+", giving up!");
			  System.exit(1);
			}
			catch(ClassNotFoundException cnfe)
			{
			  cnfe.printStackTrace();
			  System.out.println("--- AnthillPeerWorkSpace: could not read configuration file "+NEIGHBOUR_MNG_FILE_NAME+", giving up!");
			  System.exit(1);
			}
		  }
		}
	  }
	}
	NeighbourManager mng = null;
	fileNeighbourManager = new File( anthillDir, NEIGHBOUR_MNG_FILE_NAME );
	try
	{
	  fileNeighbourManager.createNewFile();
	  FileOutputStream fos = new FileOutputStream( fileNeighbourManager );
	  ObjectOutputStream oos = new ObjectOutputStream( fos );
	  mng = new NeighbourManager();
	  oos.writeObject( mng );
	  fos.close();
	}
	catch(IOException ioe)
	{
	  ioe.printStackTrace();
	  System.out.println("--- AnthillPeerWorkSpace: could not create configurationfiles, giving up!");
	  System.exit(1);
	}
	return mng;
  }

  /**
   * Method to be invoked in order to set the whole Nest up properly.
   */
  private void initialize() 
  {
  	GATE_ADV_LIFE_TIME = factory.intValue( GATE_ADV_EXPIRATION );
	TTL = factory.intValue( NEST_TTL );
	saveInfo = factory.stringValue( SERIALIZATION );
	
	//* firstly we start the platform creating the NetPeerGroup
	netPeerGroup = startJxta();
	setServices( netPeerGroup );
	
	//* creating the Manager instance
	manager = (Manager) factory.get(MANAGER_TAG);
  
	//* creating the NeighbourManager instance
	neighbourManager = createNeighbourManager();
	if ( neighbourManager == null )
	{
	  System.out.println("--- AnthillPeerWorkSpace: "+NEIGHBOUR_MNG_FILE_NAME+" hasn't been created, giving up!");
	  System.exit(1);
	}
  
	//* creating the AntFactory and the Nest instances
	antFactory = (AntFactory) factory.get(ANTFACTORY_TAG);
	nest = new Nest( factory );
	nest.setTTL( TTL );
	nest.setGateServiceID( ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID );
	nest.setAntFactory( antFactory );
	nest.setManager( manager );
	nest.setNeighbourManager( neighbourManager );
	nest.setPeerFinder( ((PeerFinder) this) );
	
	//* creating all the Advs  we need
	gateServicePipeAdv = createGateServicePipeAdvertisement( ANTHILL_GROUP_ID );
	gateServiceModSpecAdv = createGateServiceModSpecAdv( ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID, gateServicePipeAdv );
	gateServiceModImplAdv = createGateServiceModImplAdv( ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID );
	
	downloadServicePipeAdv = createDownloadServicePipeAdvertisement( ANTHILL_GROUP_ID );
	downloadServiceModSpecAdv = createDownloadServiceModSpecAdv( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID, downloadServicePipeAdv );
	downloadServiceModImplAdv = createDownloadServiceModImplAdv( ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID );
	
	nest.setDownloadServicePipeAdv( downloadServicePipeAdv );
	
	try
	{
	  anthillGroupModImplAdv = createAnthillGroupModuleImplAdvertisement( netPeerGroup.getAllPurposePeerGroupImplAdvertisement() );
	}
	catch (Exception e)
	{
	  e.printStackTrace();
	  System.exit(1);
	}
	localNestId = this.mkNestId( gateServicePipeAdv );
	nest.setId( localNestId ); 
	
	//* create the AnthillGroup instance and do all the settings
	anthillGroup = new AnthillGroup();
	anthillGroup.setNest( nest );
	anthillGroup.setGsPipeAdv( gateServicePipeAdv );
	anthillGroup.setDsPipeAdv( downloadServicePipeAdv );
	anthillGroup.setNetDiscoveryService( netDiscoService );
	anthillGroup.setNetPipeService( netPipeService );
	
	try
	 {
	  //* let's initialize the AnthillGroup, and do all the publishing
	  anthillGroup.init( netPeerGroup, ANTHILL_GROUP_ID, anthillGroupModImplAdv );
	  anthillGroup.publishGroup( ANTHILL_GROUP_NAME, ANTHILL_GROUP_DESCRIPTION);
	  netDiscoService.remotePublish( anthillGroup.getPeerGroupAdvertisement(), DiscoveryService.GROUP );
	  netDiscoService.publish( anthillGroup.getPeerGroupAdvertisement(), DiscoveryService.GROUP );
	  
	  //* now that the group has been started, let's publish our own GateServicePipeAdvertisement
	  netDiscoService.remotePublish( gateServicePipeAdv, DiscoveryService.ADV, GATE_ADV_LIFE_TIME );
	  netDiscoService.publish( gateServicePipeAdv, DiscoveryService.ADV, GATE_ADV_LIFE_TIME, GATE_ADV_LIFE_TIME );
	  
	  //* let's instantiate a the publisher thread for periodically publishing the gate service
	  //* advertisement and start it
	  publisherThread = new GateServiceAdvPublisher( netDiscoService, gateServicePipeAdv, GATE_ADV_LIFE_TIME );
	  publisherThread.start();
	 }
	catch( PeerGroupException pge )
	  {
	  pge.printStackTrace();
	  System.exit(1);
	 }
	catch(IOException ioe)
	 {
	  ioe.printStackTrace();
	  System.exit(1);
	 }   
	 
	nest.setAnthillGroup( anthillGroup );
	joinGroup( anthillGroup );
  }
  
  /**
  * Method for cleaning up before being garbage collected
  */
  protected void finalize()
  {
  }

  private int TTL;
  private long sleepTime;
  private NeighbourManager neighbourManager;
  private NestId localNestId;
  private PeerGroup netPeerGroup;
  private DiscoveryService netDiscoService;
  private PipeService netPipeService;
  private AnthillGroup anthillGroup;
  private Nest nest;
  private AntFactory antFactory;
  private Manager manager;
  private Executor executor;
  private PipeAdvertisement gateServicePipeAdv;
  private PipeAdvertisement downloadServicePipeAdv;
  private ModuleSpecAdvertisement gateServiceModSpecAdv;
  private ModuleImplAdvertisement gateServiceModImplAdv;
  private ModuleSpecAdvertisement downloadServiceModSpecAdv;
  private ModuleImplAdvertisement downloadServiceModImplAdv;
  private ModuleImplAdvertisement anthillGroupModImplAdv;
  private PeerGroupAdvertisement anthillGroupAdv;
  private Factory factory;
  private File fileNeighbourManager;
  private File anthillDir;
  private File pipeIdFile;
  private String saveInfo;
  private GateServiceAdvPublisher publisherThread;
  //private String confDir;
  
  private static String ANTHILL_GROUP_NAME = "AnthillGroup";
  private static String ANTHILL_GROUP_DESCRIPTION = "AnthillGroup-Description";
  
  private static String ANTHILL_GROUP_GATE_SERVICE_PIPE_ADV_NAME = "AnthillGroup-GateService-PipeAdv";
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_PIPE_ADV_NAME = "AnthillGroup-DownloadService-PipeAdv";
  
  private static String ANTHILL_GROUP_GATE_SERVICE_MSPEC_CREATOR = "www.cs.unibo.it";
  private static String ANTHILL_GROUP_GATE_SERVICE_MSPEC_DESCRIPTION = "JXTA-enabled-service for sending and receiving ants";
  private static String ANTHILL_GROUP_GATE_SERVICE_MSPEC_NAME = "AnthillGroup-GateService";
  private static String ANTHILL_GROUP_GATE_SERVICE_MSPEC_URI = "www.cs.unibo.it";
  private static String ANTHILL_GROUP_GATE_SERVICE_MSPEC_VERSION = "1.0";
  private static String ANTHILL_GROUP_GATE_SERVICE_MIMPL_CODE = "jxtaimpl.GateService";
  
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_CREATOR = "www.cs.unibo.it";
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_DESCRIPTION = "JXTA-enabled-service for downloading resources";
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_NAME = "AnthillGroup-DownloadService";
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_URI = "www.cs.unibo.it";
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_VERSION = "1.0";
  private static String ANTHILL_GROUP_DOWNLOAD_SERVICE_MIMPL_CODE = "jxtaimpl.DownloadService";
  
  private static String CMS_DIR_NAME = "AnthillResources";
  private static String NEIGHBOUR_MNG_FILE_NAME = "neighbours.obj";
  private static String PIPE_ID_FILE_NAME = "pipeID.obj";
  private static String SERIALIZATION = "serialization";
  //private static String CONF_DIR = "confdir";
  private static String SLEEP_TIME = "sleeptime";
  private static final String NEST_TTL = "NestTTL";
  private static final String MANAGER_TAG = "Manager";
  private static final String ANTFACTORY_TAG = "AntFactory"; 
  private static final String GATE_ADV_EXPIRATION = "GateAdvLifeTime";
  
  private static PeerGroupID ANTHILL_GROUP_ID = null;
  private static ModuleSpecID ANTHILL_GROUP_MSPEC_ID = null;
  private static ModuleClassID ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID = null;
  private static ModuleSpecID ANTHILL_GROUP_GATE_SERVICE_MSPEC_ID = null;
  private static ModuleClassID ANTHILL_GROUP_DOWNLOAD_SERVICE_MCLASS_ID = null;
  private static ModuleSpecID ANTHILL_GROUP_DOWNLOAD_SERVICE_MSPEC_ID = null; 
  private static PipeID LOCAL_GATE_SERVICE_PIPE_ID = null;
  
  //* The GateService adv will expire after 30 minutes 
  private long GATE_ADV_LIFE_TIME;
}
