/* Generated by Together */

package jxtaimpl;

import anthill.nest.AntDescriptor;

/**
 * This class reads messages from the GateServiceBuffer for processing them. The received ants will be passed to the Nest instance which implements the AntListener interface, invoking the deliver() method.
 * @author Francesco Russo (frusso@cs.unibo.it)
 * @stereotype thread
 */
public class GateServiceServer extends Thread 
{
  /**
  * Default constructor
  */
  public GateServiceServer() 
  {
    super();
    this.setName( NAME );
  }

  /**
  * Set the buffer from which message to be processed can be read
  */
  public void setBuffer(GateServiceBuffer buffer) 
  {
    gsBuffer = buffer;
  }

  public void run() 
  {
    while(true)
    {
      processMessage();
    }
  }

  /**
   * The Nest has to be set by the GateService during its initialization phase.
   * The Nest is this instance's AntListener
   */
  public void setNest(Nest nest) 
  {
    this.nest = nest;
  }

  /**
  * The message processing logic resides in this method
  */
  private void processMessage() 
  {
    //* let's read an AntDescriptor from the GateServiceBuffer
    //* this call will block until an object is available
    aDesc = (AntDescriptor) gsBuffer.read();
    
    System.out.println("--- GateServiceServer: processMessage()");
    
    try
    {
      //* now we pass this AntDescriptor to our AntListener, which is the Nest instance
      nest.deliver( aDesc );
      System.out.println("--- GateServiceServer: invoked deliver()");
    }
    catch(java.rmi.RemoteException re)
    {
      re.printStackTrace();
    }
  }

  private GateServiceBuffer gsBuffer;
  private Nest nest;
  private AntDescriptor aDesc;
  private String NAME = "GateServiceServer-Thread";
}
