/******************************************************
 * File: SharingForm.java
 * created 27-Jan-02 3:35:11 PM by root
 */

package jxtaimpl.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.border.Border;
import jxtaimpl.*;
import jxtaimpl.storage.ResourceAlreadyStoredException;

import anthill.storage.CapacityException;

public class SharingForm extends JFrame
			 implements ActionListener
{
  private JLabel resourceNameLabel;
  private JLabel resourceTypeLabel;
  private JLabel resourceDescLabel;
  private JLabel resourceLengLabel;
  private JTextField resourceTypeText;
  private JTextField resourceDescText;
  private JLabel antSpeciesLabel;
  private JTextField antSpeciesText;
  private JButton shareButton;
  private JButton cancelButton;
  
  private File file;
  private Nest nest;
  
  public SharingForm(Nest localNest, File file) 
  {
	this.file = file;
	nest = localNest;
  
	setTitle("Anthill&Jxta: Resource Form");
	setSize(400,300);
	setResizable(false);
	addWindowListener( new WindowAdapter()
					  {
				public void windowClosing( WindowEvent e)
				{
				  dispose();
				}
			 }
			);
	resourceNameLabel = new JLabel("Name: "+file.getName());
	resourceTypeLabel = new JLabel("Type: ");
	resourceDescLabel = new JLabel("Description: ");
	resourceLengLabel = new JLabel("Length: "+file.length()+" byte(s)");
	antSpeciesLabel = new JLabel("Ant Species: ");
	resourceTypeText = new JTextField();
	resourceDescText = new JTextField();
	antSpeciesText = new JTextField();
	shareButton = new JButton("Share");
	shareButton.addActionListener( this );
	cancelButton = new JButton("Cancel");
	cancelButton.addActionListener( this );
	
	JPanel panel = new JPanel();
	GridBagLayout lay = new GridBagLayout();
	Border etched = BorderFactory.createEtchedBorder();
	Border titled = BorderFactory.createTitledBorder( etched, "Resource Info");
	panel.setLayout(lay);
	panel.setBorder(titled);
	
	panel.add(resourceNameLabel, createConstraints(0,0,3,1,1,1));
	panel.add(resourceTypeLabel, createConstraints(0,1,1,1,1,1));
	panel.add(resourceTypeText, createConstraints(0,2,3,1,1,1));
	panel.add(resourceDescLabel, createConstraints(0,3,1,1,1,1));
	panel.add(resourceDescText, createConstraints(0,4,3,1,1,1));
	panel.add(resourceLengLabel, createConstraints(0,5,3,1,1,1));
	panel.add(antSpeciesLabel, createConstraints(0,6,1,1,1,1));
	panel.add(antSpeciesText, createConstraints(0,7,1,1,1,1));
	
	getContentPane().add(panel, "Center");
	
	panel = new JPanel();
	panel.add( shareButton );
	panel.add( cancelButton );
	
	getContentPane().add(panel, "South");
  }
  
  public void actionPerformed ( ActionEvent evt )
  {
	Object src = evt.getSource();
	
	if ( src == shareButton )
	{
	  String type = null;
	  String desc = null;
	  String species = null;
	  
	  type = resourceTypeText.getText().trim();
	  desc = resourceDescText.getText().trim();
	  species = antSpeciesText.getText().trim();
	  
	  if ( (type.length()==0) || (desc.length()==0) || (species.length()==0) )
	  {
		JOptionPane.showMessageDialog( this, "Fill in the form, please", "Anthill&Jxta: Missing data", JOptionPane.ERROR_MESSAGE);
	  }
	  else
	  {
		try
		{
		  nest.shareResource( file, file.getName(), type, desc, species ); 
		  JOptionPane.showMessageDialog( this, "Done!", "Anthill&Jxta: Shared!", JOptionPane.INFORMATION_MESSAGE);
		  
		  //RequestId reqId = new jxtaimpl.ids.RequestId( nest.getNestId() );
		  //String keys = file.getName()+" "+type+" "+desc;
		  //nest.insRequest( reqId, keys );
		  
		  this.dispose();
		}
		catch( SpeciesNotFoundException snfe )
		{
			JOptionPane.showMessageDialog( this, "Wrong ant species.", "Anthill&Jxta: Wrong Data", JOptionPane.ERROR_MESSAGE);  
		}
		catch(ResourceAlreadyStoredException rase)
		{
		    JOptionPane.showMessageDialog( this, "The resource seems to be already in, sorry.", "Anthill&Jxta: IOException", JOptionPane.ERROR_MESSAGE);    
		} 
		catch(CapacityException ce)
		{
		    JOptionPane.showMessageDialog( this, "Not enough space available, sorry.", "Anthill&Jxta: IOException", JOptionPane.ERROR_MESSAGE);    
		} 
	  }
	}
	else
	{
		this.dispose();
	}
  }
  
  private GridBagConstraints createConstraints(int gx, int gy, int gw, int gh, int wx, int wy)
	{
	GridBagConstraints c = new GridBagConstraints();

	c.gridx = gx;
	c.gridy = gy;
	c.gridwidth = gw;
	c.gridheight = gh;
	c.weightx = wx;
	c.weighty = wy;
	 c.insets = new Insets(1,1,1,1);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
  
	return c;
	}
}
