/******************************************************
 * File: AbstractForm.java
 * created 19-May-02 11:43:12 AM by root
 */


package techrep;
import java.awt.*;
import java.awt.event.*;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.border.Border;


public class AbstractForm extends JDialog
						  implements ActionListener
{
	private JLabel resourceAbstractLabel;
	private JLabel resourceContactLabel;
  	private JLabel resourceAuthorLabel;
	private JTextArea resourceAbstractText;
  	private JScrollPane scroll;
	private JTextField resourceAuthorText;
	private JTextField resourceContactText;
	
	private JButton addAuthorBttn;
	private JButton shareBttn;
	private JButton cancelBttn;
	
	private String abstr;
	private String author;
	private String contact;
	private LinkedList data;
	
	private boolean dataAvailable;
	
	public AbstractForm(JFrame parent)
	{
		super(parent,"Anthill&Jxta: Abstract Form",true);
	    data = new LinkedList();
		dataAvailable = false;
		
		//setTitle("Anthill&Jxta: Abstract Form");
		setSize(450,550);
		setResizable(true);
		addWindowListener( new WindowAdapter()
						  {
							public void windowClosing( WindowEvent e)
							{
							  dispose();
							}
				 		  }
						 );
		resourceAbstractLabel = new JLabel ("Abstract:");
		resourceContactLabel = new JLabel("Contact:");
 		resourceAuthorLabel = new JLabel("Author:");
		resourceAbstractText = new JTextArea(20,20);
		resourceAbstractText.setWrapStyleWord(true);
		scroll = new JScrollPane( resourceAbstractText );
		resourceAuthorText = new JTextField();
		resourceContactText = new JTextField();
		
		addAuthorBttn = new JButton("AddAuthor");
		addAuthorBttn.addActionListener( this );
		shareBttn = new JButton("Share");
		shareBttn.addActionListener( this );
		cancelBttn = new JButton("Cancel");
		cancelBttn.addActionListener( this );
		
		JPanel panel = new JPanel();
		GridBagLayout lay = new GridBagLayout();
		Border etched = BorderFactory.createEtchedBorder();
		Border titled = BorderFactory.createTitledBorder( etched, "Technical Report Info");
		panel.setLayout(lay);
		panel.setBorder(titled);
		
		panel.add(resourceAbstractLabel, createConstraints(0,0,1,1,1,1));
		panel.add(scroll, createConstraints(0,1,3,1,1,1));
		panel.add(resourceAuthorLabel, createConstraints(0,2,1,1,1,1));
		panel.add(resourceAuthorText, createConstraints(0,3,3,1,1,1));
		panel.add(resourceContactLabel, createConstraints(0,4,1,1,1,1));
		panel.add(resourceContactText, createConstraints(0,5,3,1,1,1));
		
		getContentPane().add(panel, "Center");
		
		panel = new JPanel();
		panel.add( shareBttn );
		panel.add( cancelBttn );
		panel.add( addAuthorBttn );
		
		getContentPane().add(panel, "South");
	}
	
	public void actionPerformed(ActionEvent evt)
	{
		Object src = evt.getSource();
	
		if ( src == shareBttn )
		{
			abstr = resourceAbstractText.getText().trim();
			author = resourceAuthorText.getText().trim();
			contact = resourceContactText.getText().trim();
		    
			if( (abstr.length()==0)||(author.length()==0)||(contact.length()==0) )
			{
				JOptionPane.showMessageDialog( this, "Fill in the form, please", "Anthill&Jxta: Missing data", JOptionPane.ERROR_MESSAGE);
			}
			else
			{
				data.add("ABSTRACT:: "+abstr);
				dataAvailable = true;
				this.dispose();
			}
		}
		else
		{
			if( src == addAuthorBttn )
			{
				author = resourceAuthorText.getText().trim();
				contact = resourceContactText.getText().trim();
				
				if( ((author.length()==0) && (data.isEmpty()))||(contact.length()==0) )
				{
					JOptionPane.showMessageDialog( this, "Fill in the form, please", "Anthill&Jxta: Missing author(s) or contact(s)", JOptionPane.ERROR_MESSAGE);
				}
				else
				{
					data.add("AUTHOR:: "+author);
					data.add("CONTACT:: "+contact);
					resourceAuthorText.setText("");
					resourceContactText.setText("");
				}
			}
			else
			{
				this.dispose();
			}
		}
	}
	
	protected LinkedList getData()
	{
		if( dataAvailable )
		{
			return data;
		}
		return null;
	}
	
	private GridBagConstraints createConstraints(int gx, int gy, int gw, int gh, int wx, int wy)
	{
	 	GridBagConstraints c = new GridBagConstraints();
 
 		c.gridx = gx;
		c.gridy = gy;
		c.gridwidth = gw;
		c.gridheight = gh;
		c.weightx = wx;
		c.weighty = wy;
	 	c.insets = new Insets(1,1,1,1);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.WEST;
  
		return c;
	}
}
