/******************************************************
 * File: Bib2XML.java
 * created 13-May-02 5:02:03 PM by root
 */


package techrep.TrBibReader;

import java.io.StringWriter;
import java.util.*;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.*;

/**
*  This class generates an XML document containing all the information provided
*  via the <code>LinkedList</code> the constructor requires. This list has to be
*  generated by a <code>TrBibReader.Reader</code> instance.
*/
public class Bib2XML
{
	/**
	*  Default constructor
	*/
	public Bib2XML(LinkedList list)
	{
		this.list = list;
	}
	
	/**
	*  This method generates a DOM representation of the XML document containing
	*  all the information passed via the <code>LinkedList list</code>
	*/
	public Document doFinal() throws ParserConfigurationException,
							     TransformerException
	{
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		doc = db.newDocument();
		Element root = doc.createElement(ROOT);
		
		while( !list.isEmpty() )
		{
			String item = (String)list.removeFirst();
			StringTokenizer tok = new StringTokenizer( item, TOKENS );
			String el = tok.nextToken();
			//System.out.println("--- "+el);
			String el2 = item.substring( el.length()+2 );
			//System.out.println("--- "+el2);
			
			if( el.trim().equals(AUTHOR) )
			{
				Element elAuth = doc.createElement( el.trim() );
				elAuth.setAttribute("NAME", el2.trim());
				
				//* now I have to set the two attributes: Name, Contact
				String item2 = (String)list.removeFirst();
				StringTokenizer tok2 = new StringTokenizer( item2, TOKENS );
				String contact = tok2.nextToken();
				String contactVal = item2.substring( contact.length()+2 );
				//System.out.println("--- "+contact);
				//System.out.println("--- "+contactVal);

				elAuth.setAttribute( contact.trim(), contactVal.trim() );

				root.appendChild( elAuth );
			}
			else
			{
				root.setAttribute( el.trim(), el2.trim() );
			}
			
		}
		
		doc.appendChild( root );
		
		return doc;
	}
	
	/**
	*  Returns a <code>String</code> representation of the <code>Document</code> this
	*  class instace wraps
	*/
	public String toString()
	{
		try
		{
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer t = tf.newTransformer();
			DOMSource domSrc = new DOMSource( doc.getDocumentElement() );
			StringWriter sWriter = new StringWriter();
			StreamResult streamRes = new StreamResult( sWriter );
			t.transform( domSrc, streamRes );
			return sWriter.toString();
		}
		catch(TransformerException te)
		{
			te.printStackTrace();
			System.exit(0);
		}
		return null;
	}
	
	/**
	*  Prints to the standard output a <code>String</code> representation of 
	*  the <code>Document</code> this class instace wraps
	*/
	public void print() throws TransformerException
	{
		System.out.println();
		TransformerFactory tf = TransformerFactory.newInstance();
		Transformer t = tf.newTransformer();
		DOMSource domSrc = new DOMSource( doc.getDocumentElement() );
		StreamResult streamRes = new StreamResult( System.out );
		t.transform( domSrc, streamRes );
		System.out.println();
	}
	
	private LinkedList list;
	private Document doc;
	private static String TOKENS = "::";
	private static String AUTHOR = "AUTHOR";
	
	private static String ROOT = "TrBib";
}
