/******************************************************
 * File: Reader.java
 * created 13-May-02 3:32:56 PM by root
 */

package techrep.TrBibReader;

import java.io.*;
import java.util.*;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;

/**
*  This class reads <code>bib</code> textual files and translates them
*  into XML metadata
*/
public class Reader
{
	/**
	*  Constructor. The <code>String path</code> argument is the path to 
	*  the <code>bib</code> file
	*/
	public Reader(String path) throws FileNotFoundException
	{
		fileReader = new FileReader( path );
		buffReader = new BufferedReader( fileReader );
		docList = new Vector();
		docAsStringList = new Vector();
	}
	
	/**
	*  Constructor. The <code>File file</code> argument is the File instace 
	*  pointing to the actual <code>bib</code> file
	*/
	public Reader(File file) throws FileNotFoundException
	{
		fileReader = new FileReader( file );
		buffReader = new BufferedReader( fileReader );
		docList = new Vector();
		docAsStringList = new Vector();
	}
	
	/**
	*  This method parses the <code>bib></code> file specified with the
	*  constructor, and generates the associated set of XML metadata
	*/
	public void parse() throws BibFormatException,
							   ParserConfigurationException,
							   TransformerException
	{
		String txtLine;
	
		try
		{
			list = new LinkedList();
			
			while( (txtLine=buffReader.readLine()) != null )
			{
				interpretLine( txtLine );
			}
		}
		catch( IOException ioe )
		{
			ioe.printStackTrace();
			System.exit(0);
		}
	}

	/**
	*  This method returns the set of XML metadata generated invoking the
	*  <code>parse</code> method, arranged in a <code>Vector</code>. The
	*  <code>Vector</code> stores istances of the <code>org.w3c.dom.Document</code>
	*  class.
	*/
	public Vector getDocuments()
	{
		return docList;
	}
	
	/**
	*  This method returns the set of XML metadata generated invoking the
	*  <code>parse</code> method, arranged in a <code>Vector</code>. The
	*  XML documents are represented as <code>String</code> instances.
	*/
	public Vector getDocumentsAsStrings()
	{
		return docAsStringList;
	}
	
	/**
	*  This private method handles the lines read from the <code>bib</code> file
	*/
	private void interpretLine(String txtLine) throws BibFormatException,
													  ParserConfigurationException,
													  TransformerException
	{
		StringTokenizer strTok = new StringTokenizer( txtLine, TOKENS );
			
		if( strTok.hasMoreTokens() )
		{
			String element = strTok.nextToken();
			String appo = txtLine.substring( element.length() );
			if( appo.startsWith(TOKENS) )
			{
				list.add( txtLine.trim() );
				if( element.trim().toUpperCase().equals(ENDTAG) )
				{
					//* create the XML representation of the bib item
					//* using the txt lines contained into the linked
					//* list and clear the linked list itself for next
					//* processing
					
					Bib2XML translator = new Bib2XML( list );
					Document doc = translator.doFinal();
						
					docList.add( doc );
					docAsStringList.add( translator.toString() );
					
					//translator.print();
					list.clear();
				}
			}
			else
			{
				if( (list.size()==0) & (!txtLine.equals("")) )
				{
					//throw new BibFormatException();
				}
				else
				{
					String content = (String) list.get(list.size()-1);
					content = content+" "+txtLine.trim();
					list.set(list.size()-1, content);
				}
			}
		}	
	}
	
	private void doFinal()
	{
		while( !list.isEmpty() )
		{
			String item = (String)list.removeFirst();
			System.out.println(item);
		}
	}

	
	private FileReader fileReader;
	private BufferedReader buffReader;
	private LinkedList list;
	private Vector docList;
	private Vector docAsStringList;
	
	private static String TOKENS = "::";
	private static String ENDTAG = "END";
	private static String OTHERACCESSTAG = "OTHER_ACCESS";
	private static String ABSTRACTTAG = "ABSTRACT";
}
