/******************************************************
 * File: UnsharingForm.java
 * created 29-Jan-02 6:05:43 PM by frusso@cs.unibo.it
 */

package techrep;

import java.awt.event.*;
import java.io.StringReader;

import javax.swing.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import jxtaimpl.storage.AnthillResource;

import org.w3c.dom.*;

import anthill.nest.Nest;

public class UnsharingForm extends JFrame implements ActionListener
{
  private AnthillResource[] resources;
  private JTable table;
  private JScrollPane scroll;
  private JButton unshareButton;
  private JButton infoButton;
  private Nest localNest;
  private String ABSTRACT = "ABSTRACT";
  
  public UnsharingForm(Nest nest, AnthillResource[] res)
  {
	resources = res;
	localNest = nest;
	
	setTitle("Anthill&Jxta: Unshare Local Resources");
	setSize( 500, 300 );
	setResizable( true );
	addWindowListener( new WindowAdapter()
					  {
				public void windowClosing( WindowEvent e)
				{
				  dispose();
				}
			 }
			);
	table = new JTable( res.length+1, 3 );
	table.setValueAt( "  NAME", 0, 0);
	table.setValueAt( "  TYPE", 0, 1);
	table.setValueAt( "  LENGTH (bytes)", 0, 2);
	for(int i=0; i<res.length; i++)
	{
	  if ( res[i] != null )
	  {
		table.setValueAt( res[i].getResourceName(), i+1, 0);
		table.setValueAt( res[i].getAttribute("type"), i+1, 1);
		table.setValueAt( new Long(res[i].getContentLength()), i+1, 2);
	  }
	}
	scroll = new JScrollPane( table );
	getContentPane().add( scroll, "Center" );
	
	unshareButton = new JButton("Unshare");
	unshareButton.addActionListener( this );
	infoButton = new JButton("Read Abstract");
	infoButton.addActionListener( this );
	
	JPanel panel = new JPanel();
	panel.add( unshareButton );
	panel.add( infoButton );
	
	getContentPane().add(panel, "South");
  }
  
  /**
  *  Private method for transforming the XML representation of the thechnical
  *  report's abstract into a plain text representation
  */
  private String getDescription(String xml)
  {
  	Transformer t = null;
  	TransformerFactory tf = TransformerFactory.newInstance();
	try
	{
		t = tf.newTransformer();
	}
	catch( TransformerConfigurationException tce )
	{
		tce.printStackTrace();
	}
	StringReader strReader = new StringReader( xml );
	StreamSource source = new StreamSource( strReader );
	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	DocumentBuilder db = null;
	try
	{
		db = dbf.newDocumentBuilder();
	}
	catch( ParserConfigurationException pce )
	{
		pce.printStackTrace();
	}
	Document docum = db.newDocument();
	DOMResult result = new DOMResult( docum );
	try
	{
		t.transform( source, result );
	}
	catch( TransformerException te )
	{
		te.printStackTrace();
	}
	Element el = docum.getDocumentElement();
	String abstr = el.getAttribute( ABSTRACT );
	return abstr;
  }
  
  public void actionPerformed ( ActionEvent evt )
  {
	Object src = evt.getSource();
	
	if( src == unshareButton )
	{
	  int row = table.getSelectedRow();
	  if ( (row == -1)||(row == 0) )
	  {
		JOptionPane.showMessageDialog( this, "Select one row, please.", "Anthill&Jxta: Missing Row", JOptionPane.INFORMATION_MESSAGE);
	  }
	  else
	  {
		 row = row-1;
		if( resources[row] != null )
		{
		  ((jxtaimpl.Nest)localNest).unShare( resources[row] );
		  resources[row] = null;
		  UnsharingForm form = new UnsharingForm(localNest, resources );        
		  this.dispose();
		  form.show();
		}
		else
		{
		  JOptionPane.showMessageDialog( this, "Select a valid entry, please", "Anthill&Jxta: Missing Data", JOptionPane.INFORMATION_MESSAGE);
		}
	  }
	}
	else
	{
		if( src == infoButton )
		{
	  		int row = table.getSelectedRow();
	  		if ( (row == -1)||(row == 0) )
	  		{
				JOptionPane.showMessageDialog( this, "Select one row, please.", "Anthill&Jxta: Missing Row", JOptionPane.INFORMATION_MESSAGE);
	  		}
	  		else
	  		{
				row = row-1;
				if( resources[row] != null )
				{
		  			AbstractWindow abWin = new AbstractWindow( getDescription(resources[row].getAttribute("description")) );
		  			abWin.show();
				}
				else
				{
		  			JOptionPane.showMessageDialog( this, "Select a valid entry, please", "Anthill&Jxta: Missing Data", JOptionPane.INFORMATION_MESSAGE);
				}
	  		}
		}
	}
  }
}
